require 'torch'   -- torch
require 'nn'      -- provides a normalization operator
require 'optim'
require 'image'

if not opt then
   cmd = torch.CmdLine()
   cmd:text()
   cmd:text('Dataset Preprocessing')
   cmd:text()
   cmd:text('Options:')
   cmd:option('-measure','PKR','network name')
   cmd:option('-enable16Bit','true','Enable 16 bit precision')
   cmd:option('-size',9,'Patch size')
   cmd:text()
   opt = cmd:parse(arg or {})
end

torch.setdefaulttensortype('torch.FloatTensor') -- preprocessing requires a floating point representation 

function singleForwardConfidenceMap(confImage, mapsName, size, enable16Bit)

	print(sys.COLORS.red..'Loading confidence tensor...')

	confidence = image.load(confImage)

	local pad = math.floor(size/2)
	padder_left = nn.Padding(2, -pad, 8, 0) 
	padder_right = nn.Padding(2, pad, 8, 0) 
	padder_up = nn.Padding(1, -pad, 8, 0) 
	padder_down = nn.Padding(1, pad, 8, 0) 

	input = padder_left:forward(confidence)
	input = padder_right:forward(input)
	input = padder_up:forward(input)
	input = padder_down:forward(input)

	model:float()
	output = model:forward(input)
	print(sys.COLORS.yellow..'Confidence Tensor ready!')

	-- to save as 16 bit image (higher precision)
	if enable16Bit == 'true' then
		mapH = torch.FloatTensor(output:size()[2], output:size()[3])
		mapL = torch.FloatTensor(output:size()[2], output:size()[3])

		for i=1,output:size()[2] do
			for j=1,output:size()[3] do
				local confidence = output[1][i][j] * (256*256-1)
				mapH[i][j] = (confidence / 256) / 256
				mapL[i][j] = (confidence % 256) / 256
			end
		end

		-- saving 16 bit as higher and lower bytes
		image.save(mapsName..'_H.png', mapH)
		image.save(mapsName..'_L.png', mapL)

		-- merging higher and lower bytes
		os.execute('./merge '..mapsName..'_H.png '..mapsName..'_L.png '..mapsName)
		os.execute('rm '..mapsName..'*H.png')
		os.execute('rm '..mapsName..'*L.png')
		-- invoke ./merge to produce 16 bit confidence map
	else 

		-- saving as 8 bit image (losing precision)
		image.save(mapsName, output[1])
	end	

print(sys.COLORS.green..'Confidence Tensor saved!')
end

-- main 
model = torch.load('NETS/cpu/'..opt.measure..'.net')
singleForwardConfidenceMap('example_KITTI_000000_ad/'..opt.measure..'.png', 'example_KITTI_000000_ad/'..opt.measure..'++.png', opt.size, opt.enable16Bit)




